{#  From basic/globaltoc.html

    Sphinx sidebar template: global table of contents.

    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.
#}
<h3><a href="usermodules.html">User Modules</a></h3>
{{ toctree(includehidden=True) }}
