{% extends "!layout.html" %}

{%- block extrahead %}
    <meta name="description" content="Python module for natural units">
    <meta name="keywords" content="quantity, calculus, quantities, unit, conversion, natural, SI, physics, dimension">
    <meta name="author" content="Kevin Davies">
    <link rel="icon" type="image/png" href="_static/favicon-196x196.png" sizes="196x196">
    <link rel="icon" type="image/png" href="_static/favicon-160x160.png" sizes="160x160">
    <link rel="icon" type="image/png" href="_static/favicon-96x96.png" sizes="96x96">
    <link rel="icon" type="image/png" href="_static/favicon-16x16.png" sizes="16x16">
    <link rel="icon" type="image/png" href="_static/favicon-32x32.png" sizes="32x32">
{{ super() }}
{% endblock %}
