#!/usr/bin/python
"""Aliases for constants and units of length, with support for prefixes
"""
# pylint: disable=I0011, E0611

from natu.groups import _update_module
from ..units import _units, m

_update_module(__name__, _units, m.dimension)
